/*
 * TNET		A server program for MINIX which implements the TCP/IP
 *		suite of networking protocols.  It is based on the
 *		TCP/IP code written by Phil Karn et al, as found in
 *		his NET package for Packet Radio communications.
 *
 *		Definitions for the FTP protocol (see RFC 765).
 *
 * Version:	@(#)arpa/ftp.h		1.00		07/02/92
 *
 * Authors:	Original taken from BSD 4.3/TAHOE.
 *		Fred N. van Kempen, <waltje@uwalt.nl.mugnet.org>
 */
#ifndef _ARPA_FTP_H
#define _ARPA_FTP_H

/* Reply codes. */
#define PRELIM		1	/* positive preliminary			*/
#define COMPLETE	2	/* positive completion			*/
#define CONTINUE	3	/* positive intermediate		*/
#define TRANSIENT	4	/* transient negative completion	*/
#define ERROR		5	/* permanent negative completion	*/

/* Type codes. */
#define	TYPE_A		1	/* ASCII				*/
#define	TYPE_E		2	/* EBCDIC				*/
#define	TYPE_I		3	/* image				*/
#define	TYPE_L		4	/* local byte size			*/

/* Form codes. */
#define	FORM_N		1	/* non-print				*/
#define	FORM_T		2	/* telnet format effectors		*/
#define	FORM_C		3	/* carriage control (ASA)		*/

/* Structure codes. */
#define	STRU_F		1	/* file (no record structure)		*/
#define	STRU_R		2	/* record structure			*/
#define	STRU_P		3	/* page structure			*/

/* Mode types. */
#define	MODE_S		1	/* stream				*/
#define	MODE_B		2	/* block				*/
#define	MODE_C		3	/* compressed				*/

/* Record Tokens. */
#define	REC_ESC		'\377'	/* Record-mode Escape			*/
#define	REC_EOR		'\001'	/* Record-mode End-of-Record		*/
#define REC_EOF		'\002'	/* Record-mode End-of-File		*/

/* Block Header. */
#define	BLK_EOR		0x80	/* Block is End-of-Record		*/
#define	BLK_EOF		0x40	/* Block is End-of-File			*/
#define BLK_ERRORS	0x20	/* Block is suspected of cont. errors	*/
#define	BLK_RESTART	0x10	/* Block is Restart Marker		*/

#define	BLK_BYTECOUNT	2	/* Bytes in this block			*/

#endif /* ARPA_FTP_H */
